"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var utils_1 = require("../utils");
var validators_1 = require("../validators");
var filters_1 = require("../filters");
function validatorFactory(rules) {
    return function (value, answers) {
        var _a;
        var breaking = (_a = answers.breaking, (_a !== null && _a !== void 0 ? _a : ''));
        return validators_1.validate([
            {
                value: value + breaking,
                rule: rules['footer-max-length'],
                validator: validators_1.maxLengthValidator,
                message: function (length) { return 'Footer maximum length of ${length} has been exceeded'; }
            },
            {
                value: value + breaking,
                rule: rules['footer-min-length'],
                validator: validators_1.minLengthValidator,
                message: function (length) { return "Footer minimum length of " + length + " has not been met"; }
            }
        ]);
    };
}
exports.validatorFactory = validatorFactory;
function filterFactory(rules) {
    return function (value) {
        return utils_1.pipeWith(value, function (v) { return filters_1.leadingBlankFilter(v, rules['footer-leading-blank']); }, function (v) { return filters_1.maxLineLengthFilter(v, rules['footer-max-line-length']); });
    };
}
exports.filterFactory = filterFactory;
function breakingChangeMessageFactory(rules) {
    return function () {
        var maxLength = utils_1.valueFromRule(rules['footer-max-length']);
        if (!maxLength) {
            return "Describe the breaking changes:\n";
        }
        return "Describe the breaking changes:\n (max " + maxLength + " chars):\n";
    };
}
exports.breakingChangeMessageFactory = breakingChangeMessageFactory;
function issuesMessageFactory(rules) {
    return function () {
        var maxLength = utils_1.valueFromRule(rules['footer-max-length']);
        if (!maxLength) {
            return "List issues fixed:\n";
        }
        return "List issues fixed:\n (max " + maxLength + " chars):\n";
    };
}
exports.issuesMessageFactory = issuesMessageFactory;
function isFixCommit(answers) {
    var _a, _b;
    return _b = ((_a = answers) === null || _a === void 0 ? void 0 : _a.type) == 'fix', (_b !== null && _b !== void 0 ? _b : false);
}
function issuesTransformerFactory(rules) {
    return function (value, answers) {
        var _a;
        var breaking = (_a = answers.breaking, (_a !== null && _a !== void 0 ? _a : ''));
        var footerMaxLength = utils_1.valueFromRule(rules['footer-max-length']);
        if (footerMaxLength) {
            return utils_1.maxLengthTransformerFactory(footerMaxLength - breaking.length)(value);
        }
        return value;
    };
}
exports.issuesTransformerFactory = issuesTransformerFactory;
function footerMaker(questions, rules) {
    var breakingQuestions = [
        {
            type: 'confirm',
            name: 'isBreaking',
            message: 'Are there any breaking changes?',
            default: false
        },
        {
            type: 'input',
            name: 'breaking',
            message: breakingChangeMessageFactory(rules),
            when: function (answers) { return !!answers.isBreaking; },
            validate: validatorFactory(rules),
            transformer: utils_1.maxLengthTransformerFactory(utils_1.valueFromRule(rules['footer-max-length'])),
            filter: filterFactory(rules)
        },
        {
            type: 'confirm',
            name: 'isIssue',
            message: 'Does this fix any issues?',
            when: function (answers) { return !isFixCommit(answers); },
            default: false
        },
        {
            type: 'input',
            name: 'issue',
            message: issuesMessageFactory(rules),
            when: function (answers) { return isFixCommit(answers) || !!answers.isIssue; },
            validate: validatorFactory(rules),
            transformer: issuesTransformerFactory(rules),
            filter: filterFactory(rules)
        }
    ];
    return tslib_1.__spreadArrays(questions, breakingQuestions);
}
exports.footerMaker = footerMaker;
